@@ -8,6 +8,7 @@
 #' Association, Vol. 88, No. 421, pp. 284-297
 #'
 "rsales"
+rsales <- NULL
 
 #' Series C Chemical Process Temperature Readings: Every Minute.
 #'
@@ -18,6 +19,7 @@
 #' Analysis: Forecasting and Control. John Wiley & Sons, Hoboken.
 #' 
 "seriesC"
+seriesC <- NULL
 
 #' Gas furnace data
 #' 
@@ -34,6 +36,7 @@
 #' Analysis: Forecasting and Control. John Wiley & Sons, Hoboken.
 #' 
 "seriesJ"
+seriesJ <- NULL
 
 #' Wisconsin Telephone Company
 #'
@@ -52,4 +55,18 @@
 #' Economics, The RAND Corporation, vol. 2(2), pages 515-541, Autumn.
 #'
 #' @source \url{https://drive.google.com/file/d/1LP8aMIQewMrxgOlrg9rN3eWHhZuUsY8K/view?usp=sharing}
-"Wtelephone"
\ No newline at end of file
+"Wtelephone"
+Wtelephone <- NULL
+
+#' Monthly Retail Sales: Building Material and Supplies Dealers (NAICS 4441)
+#'
+#' Monthly U.S. retail sales for building material and supplies dealers
+#' (NAICS code 4441), in millions of dollars, seasonally adjusted.
+#' Source: U.S. Census Bureau, Monthly Retail Trade Survey (via FRED).
+#'
+#' @format A time series object of class \code{ts} with frequency 12,
+#' starting in January 1992 and ending in December 2024.
+#' @references U.S. Census Bureau, Monthly Retail Trade Survey.
+#' FRED series code: MRTSSM4441USS.
+"BuildingMat"
+BuildingMat <- NULL
